@@ -1391,6 +1391,10 @@ int FileBuffer::reload(string filename, bool async)
 		if (g_fs_data.need_small_mem(filename))
 			m_allocate_way = ALLOCATION_WAYS::SEGMENT;
 
+		if(m_async_thread.joinable())
+                        m_async_thread.join();
+
+		m_dataflags = 0;
 		m_async_thread = thread(&FS_DATA::load, &g_fs_data, filename, shared_from_this());
 	}
 	else
@@ -1709,7 +1713,7 @@ std::shared_ptr<DataBuffer> FileBuffer::request_data(size_t offset, size_t sz)
 			return p;
 	}
 
-	if (sz == UINT64_MAX)
+	if (sz == SIZE_MAX)
 		sz = size() - offset;
 
 	p->resize(sz);
