@@ -164,7 +164,7 @@ public:
 	SDPWriteCmd(char*p);
 
 	int run(CmdCtx *p) override;
-	int run(CmdCtx *p, void *buff, size_t size, uint32_t addr);
+	int run(CmdCtx *p, void *buff, size_t size, uint32_t addr, bool validate = true);
 
 private:
 	uint32_t m_download_addr;
@@ -176,6 +176,7 @@ private:
 	bool m_bskipspl = false;
 	bool m_bskipfhdr = false;
 	bool m_bscanterm = false;
+	bool m_barebox_bl33 = false;
 };
 
 class SDPJumpCmd : public SDPCmdBase
@@ -212,7 +213,11 @@ public:
 	int run(CmdCtx *p) override;
 
 private:
+	bool is_barebox_img(void);
+	int load_barebox(CmdCtx *ctx);
+
 	bool m_clear_dcd = false;
 	uint32_t m_dcd_addr = 0;
 	bool m_nojump = false;
+	bool m_barebox = false;
 };
